import * as React from 'react'
import { useUser } from '@/utils/useUser'
import { useRouter } from 'next/router'
import Button from '@mui/material/Button'
import TextField from '@mui/material/TextField'
import Dialog from '@mui/material/Dialog'
import DialogActions from '@mui/material/DialogActions'
import DialogContent from '@mui/material/DialogContent'
import DialogContentText from '@mui/material/DialogContentText'
import DialogTitle from '@mui/material/DialogTitle'
import Grid from '@mui/material/Unstable_Grid2'
import { DeleteUser } from '@/api/user.interface'
import { EmailVerifyCodeScene } from '@/api/email.interface'
import * as yup from '@/utils/validation'
import * as api from '@/api'
import { useCountdown, COUNTDOWN_SECONDS } from '@/utils/useCountdown'
import { toast } from 'react-toastify'
import { useForm, SubmitHandler } from 'react-hook-form'
import { yupResolver } from '@hookform/resolvers/yup'

export function DeleteUser() {
  const { user } = useUser()
  const router = useRouter()
  const [open, setOpen] = React.useState(false)
  const [confirm, setConfrim] = React.useState(false)
  const { countdown, setCountdown } = useCountdown()

  const hookForm = useForm<DeleteUser>({
    mode: 'onTouched',
    defaultValues: {
      email: user!.email,
    },
    resolver: yupResolver(
      yup.object({
        verifyCode: yup.verifyCodeSchema,
        password: yup.string().required('请输入密码'),
      }),
    ),
  })
  const onSubmit: SubmitHandler<DeleteUser> = async (values) => {
    await api.user.deleteUser(values)
    localStorage.removeItem('accessToken')
    toast.success('用户删除成功')
    router.push('/login')
  }

  async function sendVerifyCode() {
    setCountdown(COUNTDOWN_SECONDS)
    try {
      const res = await api.email.sendEmailVerifyCode({
        email: hookForm.getValues('email'),
        scene: EmailVerifyCodeScene.deleteUser,
      })
      hookForm.setValue('token', res.data.token)
    } catch (err) {
      setCountdown(0)
    }
  }

  return (
    <>
      <Button color="error" onClick={() => setOpen(true)}>
        删除用户
      </Button>

      <Dialog
        open={open}
        fullWidth
        maxWidth="xs"
        onClose={() => setOpen(false)}
      >
        <DialogTitle>删除用户</DialogTitle>
        <DialogContent dividers>
          <Grid container spacing={[2, 1]}>
            <Grid xs={12}>
              <TextField
                {...hookForm.register('email')}
                disabled
                helperText=" "
                fullWidth
                label="邮箱"
              />
            </Grid>
            <Grid xs={8}>
              <TextField
                {...hookForm.register('verifyCode')}
                required
                error={!!hookForm.formState.errors.verifyCode}
                helperText={
                  hookForm.formState.errors.verifyCode?.message || ' '
                }
                fullWidth
                label="验证码"
              />
            </Grid>
            <Grid xs={4}>
              <Button
                size="large"
                fullWidth
                variant="contained"
                disabled={countdown > 0}
                sx={{ mt: 1 }}
                onClick={sendVerifyCode}
              >
                {countdown > 0 ? countdown : '发送验证码'}
              </Button>
            </Grid>
            <Grid xs={12}>
              <TextField
                {...hookForm.register('password')}
                required
                error={!!hookForm.formState.errors.password}
                helperText={hookForm.formState.errors.password?.message || ' '}
                fullWidth
                label="密码"
                type="password"
                autoComplete="current-password"
              />
            </Grid>
          </Grid>
        </DialogContent>
        <DialogActions>
          <Button onClick={() => setOpen(false)}>取消</Button>
          <Button color="error" onClick={() => setConfrim(true)}>
            删除
          </Button>
        </DialogActions>
      </Dialog>

      {/* 二次确认对话框 */}
      <Dialog maxWidth="xs" open={confirm}>
        <DialogTitle>确定删除？</DialogTitle>
        <DialogContent>
          <DialogContentText>
            该用户的所有数据将被不可逆地抹除！
          </DialogContentText>
        </DialogContent>
        <DialogActions>
          <Button onClick={() => setConfrim(false)}>取消</Button>
          <Button color="error" onClick={hookForm.handleSubmit(onSubmit)}>
            确定删除
          </Button>
        </DialogActions>
      </Dialog>
    </>
  )
}
